// Filename 		: BACKGRND.H
// Coded by 		: Scott Evans
// Created/Modified : 1/3/98
// Description		: Header file for Bouncer 2 maps


#ifndef _BACKGRND_H_
#define _BACKGRND_H_


#ifdef YAROZE
 
#include <libps.h>
#include <yaroze.h>

#else

#include <sys/types.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>
#include <libetc.h>
#include <libsn.h>
#include <memory.h>
#include <stdio.h>

#endif

#include <mytypes.h>
#include <objects.h>
#include <fixedp.h>
#include <printfm.h>
#include "gobjects.h"


// Values to move the background up or down

enum
{
	UP,
	DOWN
};


// Dimensions of biggest map 20*400 = 8000 bytes

#define MAX_MAP_COLUMNS	20
#define MAX_MAP_ROWS	400

//#define MAX_MAP_BLOCK_TYPES	128
//#define BG_OT_LENGTH		7


typedef struct
{
	//GsOT *ot[2];			// Ordering table used for background
	OBJECT *block;			// Object used to draw background
	SprImgInfo *info;		// Information about textures
	RECT area;				// Area of background on screen
	u_word cx,cy;			// Position of CLUT
	u_word rows,columns;	// Size of map in rows and columns
	u_word xoff,yoff;		// Offsets into map
	u_word no_diamonds;		// Number of diamonds on level
	u_byte bw,bh;			// Block width and height (assume all blocks are the same size)
	byte *data;				// Data for each level
}BACKGRND;


// Level bank file header structure

typedef struct
{
	u_word nlevels;			// number of levels in bank
	u_long offset;			// offset from start of header to 1st parameter block
}LevelHeader;


// Level parameter block structure

typedef struct
{
	u_word nrows;			// size of level in rows
	u_word nclms;			// Size of level in columns
	u_word ndiamonds;		// Number of diamonds on level
	u_long offset;			// offset to level data (from start of parameter block)
}LevelPblk;


// Macro			: _GetNoLevels()
// Coded by			: Scott Evans
// Created/Modified	: 22/9/97
// Description		: Find number of levels in a level bank

// Parameters		: h - pointer to level bank

// Returns			: Number of levels

// Notes			: None


#define _GetNoLevels(h)\
	((h)->nlevels)
	

// Macro			: _GetLevelPblks()
// Coded by			: Scott Evans
// Created/Modified	: 22/9/97
// Description		: Find address of 1st parameter block

// Parameters		: h - pointer to level bank

// Returns			: Pointer to 1st level parameter block

// Notes			: None


#define _GetLevelPblks(h)\
	((LevelPblk *)(((u_byte *)(h))+(h)->offset))


// Macro			: _GetLevelPblk()
// Coded by			: Scott Evans
// Created/Modified	: 22/9/97
// Description		: Find address of nth parameter block

// Parameters		: h - pointer to level bank
//					  n - number of parameter block to find
	
// Returns			: Pointer to nth parameter block

// Notes			: None
	
	
#define _GetLevelPblk(h,n)	(_GetLevelPblks(h)+(n))
	
	
// Macro			: _GetLevelSize()
// Coded by			: Scott Evans
// Created/Modified	: 22/9/97
// Description		: Find size of a level in bytes

// Parameters		: h - pointer to level bank
//					  l - number of level
	
// Returns			: Size of level in bytes

// Notes			: None
	
	
#define _GetLevelSize(h,l)\
	((_GetLevelPblk(h,l)->nrows*(_GetLevelPblk(h,l)->nclms))
	

// Macro			: _GetLevelData()
// Coded by			: Scott Evans
// Created/Modified	: 22/9/97
// Description		: Find pointer to level data

// Parameters		: h - pointer to level bank
//					  l - number of level
	
// Returns			: Pointer to level data
 
// Notes			: None
	
	
#define _GetLevelData(h,l)\
	(((_byte *)(_GetLevelPblk(h,l))+(_GetLevelPblk(h,l))->offset)
	
	
// Size of above structures

#define LevelHeaderSize	sizeof(LevelHeader)
#define LevelPblkSize	sizeof(LevelPblk)


void InitBackground(SPRITE_DATA_INFO *,BACKGRND *,RECT *);
void SortBackground(BACKGRND *,GsOT *,u_word);
void CopyLevelToBuffer(BACKGRND *,LevelHeader *,u_word);
u_word MoveBackground(BACKGRND *,u_byte);
byte GetMapBlock(BACKGRND *,u_word,u_word);
void SetMapBlock(BACKGRND *,u_word,u_word,byte);
void InitLevel(BACKGRND *,LevelHeader *,u_word);
void CheckMapRow(BACKGRND *,u_word);
void MoveBackgroundObjects(word);
void MoveObjectsOnMap(OBJECT_LIST_HEADER *,word);


#endif
