// Filename 		: BOUNCER.H
// Coded by 		: Scott Evans
// Created/Modified : 6/3/98
// Description		: Bouncer 2


#ifndef _BOUNCER_H_
#define _BOUNCER_H_


#ifdef YAROZE

#include <libps.h>
#include <yaroze.h>

#else

#include <sys/types.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>
#include <libetc.h>
#include <libsn.h>

#endif

#include <rand.h>
#include <stdio.h>
#include <memory.h>
#include <dcache.h>
#include <mytypes.h>
#include <mylib.h>
#include <stosspr.h>
#include <objects.h>
#include <sprites.h>
#include <vsync.h>
#include <font.h>
#include <fixedp.h>
#include <sound.h>
#include <printfm.h>
#include <intro.h>

#include "backgrnd.h"
#include "sprmap.h"
#include "gobjects.h"
#include "handlers.h"
#include "hscore.h"
#include "message.h"
#include "loadsave.h"
#include "menu.h"
#include "cheats.h"
#include "data.h"


// Software name and versin number

#define SW_NAME		"BOUNCER 2"
#define SW_VERSION	1


// Different levels of brightness used in game

enum
{
	GAME_TEXT_BRIGHTNESS,
	GAME_MIN_BRIGHTNESS,
	GAME_MAX_BRIGHTNESS,
	GAME_NORMAL_BRIGHTNESS,
	GAME_DEMO_BRIGHTNESS,
	GAME_UP_BLOCK_BRIGHTNESS,
	GAME_EFFECTS_BRIGHTNESS,
	GAME_RED_BRIGHTNESS=GAME_EFFECTS_BRIGHTNESS,
	GAME_GREEN_BRIGHTNESS,
	GAME_BLUE_BRIGHTNESS,
	GAME_YELLOW_BRIGHTNESS,
	GAME_WHITE_BRIGHTNESS,
	GAME_PURPLE_BRIGHTNESS,
	GAME_BLUEGREEN_BRIGHTNESS,
	GAME_INVISIBLE_BLOKE_BRIGHTNESS,
	GAME_TELEPORT_BLOKE_BRIGHTNESS,
	GAME_MESSAGE_BRIGHTNESS,
	GAME_MENU_DISABLED_BRIGHTNESS,
	GAME_DEMO_YELLOW_BRIGHTNESS,
	GAME_EXTRA_BRIGHTNESS,
	GAME_REVERSE_BRIGHTNESS,
	GAME_SHIELD_BRIGHTNESS,
	GAME_DECOY_BRIGHTNESS,
	GAME_MOVE_BLOKE_BRIGHTNESS
};


enum
{
	GAME_OVER_INITIALISE,
    GAME_OVER_HIGHSCORE,
    GAME_OVER_START_SELECTION,
    GAME_OVER_ENTER_NAME,
	GAME_OVER_FINISHED_EDITING,
    GAME_OVER_END,
	GAME_OVER_CLEAR_OBJECTS
};


enum
{
	PS_INITIALISE,
	PS_MOVE_SCREEN,
	PS_END
};


// Width and height of screen

#define SCREEN_WIDTH	320

#ifdef PAL
#define SCREEN_HEIGHT	256
#else
#define SCREEN_HEIGHT	240
#endif


// Screen offsets

#define SCREEN_X_OFFSET	4
#define SCREEN_Y_OFFSET	16


// Level stuff

#ifdef PAL
#define MAX_MAP_SCREEN_ROWS		16
#else
#define MAX_MAP_SCREEN_ROWS		14
#endif

#define MAX_MAP_SCREEN_COLUMNS	20
#define MAX_MAP_PACKETS			(MAX_MAP_SCREEN_ROWS*MAX_MAP_SCREEN_COLUMNS)


// Size of the play area

#define GAME_SCREEN_X	0
#define GAME_SCREEN_Y	0
#define GAME_SCREEN_W	(MAX_MAP_COLUMNS<<4)
#define GAME_SCREEN_H	(MAX_MAP_SCREEN_ROWS<<4)


// Ordering table and packet buffer stuff

#define OT_LENGTH			2
#define MAX_PACKETS			(MAX_MAP_PACKETS+1000)
#define MAX_PACKET_SIZE		24

#define MAX_OT_DEPTH		((1<<OT_LENGTH)-1)
#define BACKGROUND_OT_DEPTH	MAX_OT_DEPTH
#define MENUS_OT_DEPTH		(MAX_OT_DEPTH-1)
#define HIGHSCORES_OT_DEPTH	(MAX_OT_DEPTH-1)
#define OBJECTS_OT_DEPTH	(MAX_OT_DEPTH-1)
#define MESSAGES_OT_DEPTH	(MAX_OT_DEPTH-1)
#define DEBUG_OT_DEPTH		0

// Controller button repeat rate

#define BUTTON_REPEAT_RATE	50


// Texture information

#define MAX_TEXTURES	300


// Default texture page and CLUT positions

#define TPAGE_X		SCREEN_WIDTH
#define TPAGE_Y		0
#define CLUT_X		(1024-16)
#define CLUT_Y		0


// Font texture page and CLUT positions

#define FONT_TPAGE_X			TPAGE_X
#define FONT_TPAGE_Y			TPAGE_Y
#define FONT_CLUT_X				CLUT_X
#define FONT_CLUT_Y				CLUT_Y

// Map blocks texture page and CLUT positios

#define MAP_BLOCKS_TPAGE_X		(TPAGE_X+256)
#define MAP_BLOCKS_TPAGE_Y		TPAGE_Y
#define MAP_BLOCKS_CLUT_X		CLUT_X
#define MAP_BLOCKS_CLUT_Y		(FONT_CLUT_Y+1)

// Game sprites

#define GAME_SPRITES_TPAGE_X	TPAGE_X
#define GAME_SPRITES_TPAGE_Y	TPAGE_Y
#define GAME_SPRITES_CLUT_X		CLUT_X
#define GAME_SPRITES_CLUT_Y		(MAP_BLOCKS_CLUT_Y+3)


// Game stuff

#define DEFAULT_NO_LIVES	4
#define MAX_NO_LIVES		8
#define DEFAULT_NO_BOMBS	4
#define MAX_NO_BOMBS		8


// Sound effect volumes

#define SFX_DEMO_VOLUME		40
#define SFX_MENU_VOLUME		SS_MAX_VOLUME
#define SFX_GAME_VOLUME		SS_MAX_VOLUME


// Sound effects list

enum
{
	SFX_COLLECT_BONUS,
	SFX_OHOH,
	SFX_BLOKE_DEAD,
	SFX_MENU_MOVE,
	SFX_SIREN,
	SFX_HIT_BAT,
	SFX_MESSAGE_WOOSH,
	SFX_HIT_BLOCK,
	SFX_HIT_BLOCK2,
	SFX_HIT_BLOCK3,
	SFX_HIT_BLOCK4,
	SFX_CLOCK_TICK,
	SFX_MENU_SELECT,
	SFX_RUNNING_OUT_OF_TIME,
	SFX_HIT_INDUSTRUCTIBLE_BLOCK,
	SFX_EXPLOSION,
	SFX_MENU_BOUNCE,
	SFX_HIT_DIAMOND,
	SFX_COUNTER,
	SFX_SUPER_BLOKE,
	SFX_HIT_FLOOR,
	SFX_EXTRA_LIFE,
	SFX_TELEPORT,
	SFX_GAME_OVER,
	SFX_FIREGUN,
	SFX_EXPANDING_BLOCK,
	SFX_INVISIBLE,
	SFX_SPLIT_BLOKE,
	SFX_UP_ARROW,
	SFX_REVERSE_CONTROLS,
	SFX_MENU_CANCEL,
	SFX_CHEAT_ENABLED,
	SFX_CHEAT_ENABLED1,
	SFX_CHEAT_ENABLED2,
	SFX_INTRO_FADE,
	SFX_SHIELD,
	SFX_FILL_BONUS_BAR
};


// Max objects per level

#define MAX_FANS		30
#define MAX_GUNS		30
#define MAX_PLATFORMS	30


// Difficulty settings

enum
{
	EASY,
	MEDIUM,
	HARD
};


// Startup mode values

enum
{
	HIGHSCORES,
	MENUS,
	MESSAGES
};


// Message modes

enum
{
	MM_NONE,
	MM_POSITION_SCREEN,
	MM_ABOUT_B2,
	MM_INSTRUCTIONS,
	MM_LOADGAME,
	MM_SAVEGAME,
	MM_FORMAT,
	MM_EXITGAME
};


// Bonus levels

enum
{
	START_BONUS_LEVEL=30,
	BL_CATCH=START_BONUS_LEVEL,
	BL_COLLECT,
	BL_BLOCK
};


enum
{
	BT_NONE,
	BT_CATCH,
	BT_COLLECT,
	BT_BLOCK
};


#define MAX_BONUS_LEVELS	(BL_BLOCK-START_BONUS_LEVEL+1)

#define EXTRA_LIFE_SCORE	50000


#ifdef PAL
#define SECOND	50
#else
#define SECOND 60
#endif

#define MINUTE 60


// How often to change game modes

#define MODE_CHANGE_TIMER	(SECOND*20)

#define BLOCK_COUNTER_FULL	(-50)
#define BLOCK_STACK_SIZE	50


// Game data

typedef struct
{
	u_byte block_stack[BLOCK_STACK_SIZE+2];
	u_long block_sp;
	OBJECT *multiplier_sprite;
	OBJECT *life_icons[MAX_NO_LIVES];
	OBJECT *bomb_icons[MAX_NO_BOMBS];
	long score;	
	u_long extra_life_score;	// How mant points needed for extra life
	u_word no_levels;			// Maximum number of levels
	u_word level_no;			// Current level
	u_word start_level;			// The very 1st level
	u_word next_level;			// The level to be played after the bonus level
	u_word last_level;			// The last level played
	u_word timer;				// Used to switch between game modes
	u_word cheats_on;			// Active cheat codes
	u_byte no_lives;			// Number of lives
	u_byte no_bombs;			// Number of bombs
	u_byte difficulty;			// Game difficulty level
	u_byte mode;				// Startup mode
	u_byte demo_mode;			// Are we in demo mode
	u_byte leaving;				// Should we be leaving the game
	u_byte shake_screen;		// Should we be shaking the screen
	u_byte level_complete;		// Have we finished the level
	u_byte paused;				// Is the game paused
	u_byte bonus_multiplier;	// 2, 5 or 10 times normal bonuses
	u_byte show_intro;			// Do we run through the intro sequence
	u_byte game_over;			// Have we lost all our lives
	u_byte no_time_bonuses;		// How much extra time to give them when they run out
	u_byte message_mode;		// Which messages should be displayed
	u_byte bonus_level;			// Is the next level a bonus level
	u_byte bonus_type;			// The type of bonus level
	u_byte mc_mode;				// Save or load game
	u_byte mcard;				// Which memory card to use
	u_byte game_started;		// Has the game just started
	u_byte just_loaded;			// Just loaded a game
	u_byte kill_fireworks;		// Explode all the fireworks
	u_byte bspr_clear;			// Are the bonus sprites clear
	u_byte level_warp;			// Did we get a level warp bonus
	u_byte intro_counter;
	u_byte pad_status; 
	u_byte lspr_clear;
}GAME_DATA;


// Bloke data

typedef struct
{
	long tx,ty;					// Teleport coordinates
	word super_timer;			// Duration for super bloke
	word invisible_timer;		// Duration of invisible effect
	word reverse_timer;			// Duration of reverse controls effect
	word shield_timer;			// How long the bloke is safe
	long beam_timer;			
	OBJECT *beam;
	u_byte super;				// Flag for super bloke
	u_byte invisible;			// Invisible effect
	u_byte teleporting;			// Bloke is teleporting
	byte teleport_finished;		// Have we finished the teleport effect
	u_byte reverse;				// Reverse controls effect
	u_byte shield;				// Bloke protected from bullets
}BLOKE_DATA;


// The different types of counter

enum
{
	DIGIT_COUNTER,
	GFX_COUNTER
};


// Graphical counter

typedef struct
{
	OBJECT *bar;
	OBJECT *back;
}BAR;


// Counter type union

typedef union
{
	OBJECT_STRING string;
	BAR bar; 
}COUNTER_TYPE;


// Counter structure

typedef struct
{
	COUNTER_TYPE type;
	long counter;
	long extra;
	u_byte has_shadow;
	u_byte type_code;
	u_byte empty;
}COUNTER;


typedef struct
{
	OBJECT_STRING time_string;
	word no_seconds;
	word extra;
	u_byte going;
}LEVEL_TIMER;


#define TIMER_X			256
#define TIMER_Y			16


#define NO_SCORE_DIGITS	8
#define SCORE_POS_X		16
#define SCORE_POS_Y		16


#define LIFE_ICON_X		16
#define LIFE_ICON_Y		26


#define BOMB_ICON_X		16
#define BOMB_ICON_Y		36


#define DIAMOND_COUNTER_X			272
#define DIAMOND_COUNTER_Y			36
#define NO_DIAMOND_COUNTER_DIGITS	3


#define BONUS_COUNTER_X				272
#define BONUS_COUNTER_Y				36


#define MAX_COUNTERS 		10


#define BLOCK_POS_X	304
#define BLOCK_POS_Y	48


#define OFF_SCREEN_Y	60

#define BONUS_TIME_LIMIT	(MINUTE)


// Save game states

enum
{
	SG_START,
	SG_CHANGE_STATE,
	SG_END,
	SG_INITSAVE,
	SG_SAVE,
	SG_INITCOMPLETE,
	SG_COMPLETE
};


// Load game states

enum
{
	LG_START,
	LG_CHANGE_STATE,
	LG_END,
	LG_INITLOAD,
	LG_LOAD,
	LG_INITCOMPLETE,
	LG_COMPLETE
};


// Format memory card states

enum
{
	FC_START,
	FC_CHANGE_STATE,
	FC_END,
	FC_INITFORMAT,
	FC_FORMAT,
	FC_INITCOMPLETE,
	FC_COMPLETE
};


// Funtion prototypes

void InitGame(void);
u_word InitGameFont(u_word);
void Game(void);
void FadeAllObjects(void);
void UpdateGameMode(void);
void UpdateAndSortGameObjects(void);
u_word InitGameBackground(u_word);
void InitStartPlay(void);
void InitGameData(GAME_DATA *);
void InitLeaving(void);
u_word InitGameSprites(u_word);
OBJECT *CreateBatSprite(void);
void ShakeScreen(u_byte,u_byte,u_byte);
void PauseGame(void);
void InitLevelTimer(LEVEL_TIMER *);
void SetLevelTimer(LEVEL_TIMER *);
void StartLevelTimer(void);
void StopLevelTimer(void);
void AddToTime(word);
void InitLives(void);
void InitBombs(void);
void UpdateLevelTimer(void);
void MoveScreen(DBUFFER **);
void LevelComplete(void);
COUNTER *CreateCounter(FONT_INFO *,DVECTOR *,word,u_byte,u_byte);
void SetCounter(COUNTER *);
void UpdateCounter(COUNTER *);
void AddToCounter(COUNTER *,long);
void SubtractFromCounter(COUNTER *,long);
void CreateDiamondCounter(void);
void CreateScoreCounter(void);
void ResetBlockCount(long);
void CreateBlockCounter(void);
void KillAllObjects(void);
void MoveScoreInfoObjects(u_byte,u_byte);
void GameOver(void);
void PositionScreen(PROGRAM_DATA *);
void InitObjectLists(void);
void AboutB2(PROGRAM_DATA *);
void Instructions(PROGRAM_DATA *);
void InitBonusLevel(u_byte);
void CreateBonusLevelCounter(void);
void CreateExtraLife(void);
void CreateExtraBomb(void);
void SaveGameScreen(PROGRAM_DATA *);
void LoadGameScreen(PROGRAM_DATA *);
void FormatCardScreen(PROGRAM_DATA *);
void ResumeGame(void);
void StartGame(void);
void ExitGame(PROGRAM_DATA *);
void FlashScoreObjects(void);
void LevelSkipCheat(void);
void Version(VERSION_DATA *);
u_long LoadGameData(void);
void ShowTitleScreen(void);


#endif
