// Filename 		: CHEATS.C
// Coded by 		: Scott Evans
// Created/Modified : 17/8/98
// Description		: B2 game cheat modes


#include "cheats.h"


extern GAME_DATA gd;
extern PROGRAM_DATA pd;
extern SOUND_DATA *sounds;
extern VOLUME menu_volume;


word cheat_codes[]={
	#include "cheats.dat"
};


GAME_CHEATS gc;


// Function			: InitCheatCodes()
// Coded by			: Scott Evans
// Created/Modified	: 17/8/98
// Description		: Initialise game cheats

// Parameters		: None

// Returns			: None

// Notes			: None


void InitCheatCodes(void)
{
	// Clear out structure

	bzero((u_byte *)&gc,sizeof(GAME_CHEATS));

	// Set pointer to codes

	gc.codes=&cheat_codes[0];

	// Disable all cheats

	gd.cheats_on=CC_NONE;

	#ifdef DEBUG_INFO

	PrintFM("Completed\n");

	#endif
}


// Function			: CheckForCheats()
// Coded by			: Scott Evans
// Created/Modified	: 17/8/98
// Description		: Check for cheat codes

// Parameters		: gd - pointer to game data
//					  pd - pointer to program data

// Returns			: None

// Notes			: None

  
void CheckForCheats(void)
{
	static u_word button_pressed=0;
	static u_long timer=0;
	u_byte i;
	word code;

	if(pd.pad && !pd.buttons_down)
	{
		timer=0;
		button_pressed=pd.pad;

		if(!gc.started)
		{
			// Test first character of each code
			 
			for(i=0;i<MAX_CHEATS;i++)
			{
				code=gc.codes[((CODE_LENGTH+1)*i)+gc.next];
				
				if(button_pressed==code)
				{
					// We have a match so try other characters in code

					gc.current=i;
					gc.started=1;
					gc.next++;

					break;
				}
			 }
		}
		else
		{
			code=gc.codes[((CODE_LENGTH+1)*gc.current)+gc.next];
			
			// Check next character of code

			if(button_pressed==code)
			{
				gc.next++;

				if(gc.codes[((CODE_LENGTH+1)*gc.current)+gc.next]==CODE_TERMINATOR)
				{
					// Code matches, toggle cheat

					if(gd.cheats_on&(1<<gc.current))
					{
						// Turn cheat off if already on

						gd.cheats_on&=~(1<<gc.current);
						PlaySoundEffect(sounds,&menu_volume,SFX_MENU_CANCEL);

						#ifdef DEBUG_INFO

						sprintf(fm_string,"code %d deactivated\n",gc.current);
						PrintFM(fm_string);

						#endif
					}
					else
					{
						// Turn the cheat on

						gd.cheats_on|=(1<<gc.current);
					
						gc.started=0;
						gc.next=0;

						PlaySoundEffect(sounds,&menu_volume,SFX_CHEAT_ENABLED+(rand()%3));

						if(gd.cheats_on&CC_DISABLE_ALL)
							gd.cheats_on=CC_NONE;
						#ifdef DEBUG_INFO

						sprintf(fm_string,"code %d activated\n",gc.current);
						PrintFM(fm_string);

						#endif
					}
				}
			}
			else
			{
				// Wrong code, start again

				gc.started=0;
				gc.next=0;
			}
		}
	}
	else
	{
		// Make sure they press the next button within time limit

		timer++;
	
		if(!(timer%14))
		{
			// Start again

			gc.started=0;
			gc.next=0;
		}
	}
}
