// Filename 		: BACKGRND.C
// Coded by 		: Scott Evans
// Created/Modified : 1/3/98
// Description		: Contains functions for Bouncer 2 maps


#include "backgrnd.h"
#include "sprmap.h"


byte level_buffer[MAX_MAP_COLUMNS*MAX_MAP_ROWS];

			       
// Function			: InitBackground()
// Coded by			: Scott Evans
// Created/Modified	: 1/3/98
// Description		: Initialise a background

// Parameters		: info - pointer to sprite information for map blocks
//				  	  bg - pointer to background structure
//				 	  r - define on screen area of background 

// Returns			: None

// Notes			: None


void InitBackground(SPRITE_DATA_INFO *info,BACKGRND *bg,RECT *r)
{
	static OBJECT block;
	BRIGHTNESS b={0x80<<FP12_4,0x80<<FP12_4,0x80<<FP12_4};
	
	// Clear object structure
	
	bzero(&block,sizeof(OBJECT));
	
	// Set object type to GsSPRITE
	
	block.type_code=GsSPRITE_OBJECT;
									  
	// Allow the object to be faded
	
	block.allow_fading=1;
	
	// Set up sprite used to draw map
	
	SetGsSPRITE(&block.type.gss,0,0,0,info->offsets[FIRST_MAP_BLOCK_SPRITE].w,info->offsets[FIRST_MAP_BLOCK_SPRITE].h,
		info->tpage,0,0,info->cx,info->cy+info->offsets[FIRST_MAP_BLOCK_SPRITE].clut_y,0x80,0x80,0x80,0,0,ONE,ONE,0);
	
	SetObjectsBrightness(&block,&b);
	
	bg->block=&block;

	// Set up background structure
	
	setRECT(&bg->area,r->x,r->y,r->w,r->h);
	bg->xoff=bg->yoff=0;
	bg->info=info->offsets;
	bg->rows=bg->columns=0;
	bg->no_diamonds=0;
	
	// Set blocks width and height
	
	bg->bw=info->offsets[FIRST_MAP_BLOCK_SPRITE].w;
	bg->bh=info->offsets[FIRST_MAP_BLOCK_SPRITE].h;
	
	// Set backgrounds CLUT position
	
	bg->cx=info->cx;
	bg->cy=info->cy;
	
	// Set level data buffer
	
	bg->data=&level_buffer[0];
	
	printf("InitBackground : Completed\n");
}


// Function			: SortBackground()
// Coded by			: Scott Evans
// Created/Modified	: 1/3/98
// Description		: Sort a sprite background in ordering table

// Parameters		: bg - pointer to background to draw
//				 	  ot - pointer to ordering table
//				 	  priority - priority of background in ordering table

// Returns			: None

// Notes			: None


void SortBackground(BACKGRND *bg,GsOT *ot,u_word priority)
{
	GsSPRITE *sprite;
	u_word r,c,h,w,x,y,offset;
	u_byte blockw,blockh;
	byte block_no;
	
	h=bg->area.h;
	w=bg->area.w;
	y=bg->area.y;
	
	blockw=bg->bw;
	blockh=bg->bh;
	
	sprite=&bg->block->type.gss;
		
	for(r=0;r<h;r++)
	{
		x=bg->area.x;
		offset=(r+bg->yoff)*w;
		
		for(c=0;c<w;c++)
		{
			if(!((block_no=bg->data[c+offset])&0x80))
			{
				sprite->x=x;
				sprite->y=y;
				sprite->u=bg->info[block_no].u;
				sprite->v=bg->info[block_no].v;
				sprite->cy=bg->cy+bg->info[block_no].clut_y;
				
				GsSortFastSprite(sprite,ot,priority);
			}
			
			x+=blockw;
		}
		
		y+=blockh;
	}	
}


// Function			: CopyLevelToBuffer()
// Coded by			: Scott Evans
// Created/Modified	: 2/3/98
// Description		: Fades the brightness of the background 

// Parameters		: bg - pointer to background to draw
//				  	  l - pointer to level data
//				  	  n - level number to load

// Returns			: None

// Notes			: None


void CopyLevelToBuffer(BACKGRND *bg,LevelHeader *l,u_word n)
{
	// Find parameter block for level
	
	LevelPblk *pblock=_GetLevelPblk(l,n);
	
	// Calculate size of level
	
	u_long size=pblock->nrows*pblock->nclms;
	
	// Copy level data to level buffer
	
	memcpy(&level_buffer[0],(byte *)pblock+pblock->offset,size);
	
	bg->rows=pblock->nrows;
	bg->columns=pblock->nclms;
	bg->no_diamonds=pblock->ndiamonds;
	
	printf("CopyLevelToBuffer : Level %d = %d (bytes)\n",n,size);
	printf("CopyLevelToBuffer : Rows = %d Columns = %d Diamonds = %d\n",bg->rows,bg->columns,bg->no_diamonds);
}


// Function			: MoveBackground()
// Coded by			: Scott Evans
// Created/Modified	: 3/3/98
// Description		: Moves the background a row at a time 

// Parameters		: bg - pointer to background to draw
//				  	  mode - UP or DOWN

// Returns			: Updated position in map

// Notes			: None


u_word MoveBackground(BACKGRND *bg,u_byte mode)
{
	switch(mode)
	{
		case UP:
			if(bg->yoff>0)
				bg->yoff--;
		break;
		
		case DOWN:
			if(bg->yoff<bg->rows)
				bg->yoff++;
		break;
	}
	
	return(bg->yoff);
}


// Function			: GetMapBlock()
// Coded by			: Scott Evans
// Created/Modified	: 3/3/98
// Description		: Get block number from map 

// Parameters		: bg - pointer to background
//				  	  row,column - position in map

// Returns			: Block number

// Notes			: None


byte GetMapBlock(BACKGRND *bg,u_word row,u_word column)
{
	return(bg->data[((row+bg->yoff)*bg->area.w)+column]);
}


// Function			: SetMapBlock()
// Coded by			: Scott Evans
// Created/Modified	: 3/3/98
// Description		: Set block number in map 

// Parameters		: bg - pointer to background
//				   	  row,column - position in map
//				  	  block - block number to set

// Returns			: None

// Notes			: None


void SetMapBlock(BACKGRND *bg,u_word row,u_word column,byte block)
{
	bg->data[((row+bg->yoff)*bg->area.w)+column]=block;
}


// Function			: InitLevel()
// Coded by			: Scott Evans
// Created/Modified	: 7/3/98
// Description		: Initialise a level 

// Parameters		: bg - pointer to background
//				  	  l - pointer to levels
//				  	  n - level number to initialise

// Returns			: None

// Notes			: None


void InitLevel(BACKGRND *bg,LevelHeader *l,u_word n)
{
	u_word r;
	
	CopyLevelToBuffer(bg,l,n);

	printf("InitLevel : Level %d initialised\n",n);
}
